#
# Usage: create_resource.sh project_id cluster_name namespace cluster_zone luna_nodepool
#

set -e

readonly PROJECT_ID=$1
readonly CLUSTER_NAME=$2
readonly NAMESPACE=$3
readonly CLUSTER_ZONE=$4
readonly LUNA_NODEPOOL=$5
readonly service_account_prefix=$6

: \
  ${LUNA_MANAGER_REPO:="elotl/luna-trial"} \
  ${LUNA_WEBHOOK_REPO:="elotl/luna-admission-webhook-trial"}

readonly SYSTEM_GSA=${SYSTEM_GSA:-$(echo ${CLUSTER_NAME} | cut -c1-24)-elotl}
readonly LUNA_MANAGER_SERVICE_ACCOUNT_NAME="${service_account_prefix}-manager"

if [[ "$ENABLE_WORKLOAD_IDENTITY" == "" ]]; then
  okay_to_enable_workload_identity="false"
else
  okay_to_enable_workload_identity=${ENABLE_WORKLOAD_IDENTITY}
fi


function _gcloud() {
  command gcloud --project "$PROJECT_ID" "$@"
}

function create_namespace_if_not_exist() {
  kubectl create namespace "$NAMESPACE" --dry-run=client -o yaml | kubectl apply -f -
}

function create_gsa_if_not_exist {
  local name=${1}
  echo "Creating Google service account $name"
  local already_present
  already_present=$(_gcloud iam service-accounts list --filter='name:'$name'' --format='value(name)')
  if [ -n "$already_present" ]; then
    echo "Service account $name already exists"
  else
    _gcloud iam service-accounts create "$name"
  fi
}

function add_gsa_roles {
  local project_id=${1}
  local system_gsa=${2}
  roles="roles/container.developer roles/compute.instanceAdmin.v1 roles/iam.serviceAccountUser roles/container.admin"
  echo "Adding roles: $roles to Google service account $system_gsa"
  for role in $roles; do
    _gcloud projects add-iam-policy-binding "$project_id" \
        --member="serviceAccount:$system_gsa@$project_id.iam.gserviceaccount.com" \
        --role="$role" \
        > /dev/null
  done
}

function bind_gsa_and_ksa {
  local gsa=${1}
  local ksa=${2}

  _gcloud iam service-accounts add-iam-policy-binding "$gsa"@"$PROJECT_ID".iam.gserviceaccount.com \
    --member="serviceAccount:$PROJECT_ID.svc.id.goog[$NAMESPACE/$ksa]" \
    --role="roles/iam.workloadIdentityUser" \
    > /dev/null
}

function ask_permission_to_enable_workload_identity {
  if [ "$okay_to_enable_workload_identity" == "false" ]; then
    echo "Workload identity is not enabled at cluster $CLUSTER_NAME's nodepool $LUNA_NODEPOOL. Luna won't work without this enabled."
    echo "Enable workload identity for cluster $CLUSTER_NAME's nodepool $LUNA_NODEPOOL? This can take up to 20 minutes."
        while true; do
            read -rp "Proceed with enabling workload identity (Y/N): " choice
            case $choice in
                [Yy])
                    okay_to_enable_workload_identity="true"
                    break
                    ;;
                [Nn])
                    echo "Script execution cancelled by the user."
                    exit 1
                    ;;
                *)
                    echo "Invalid input. Please enter Y or N."
                    ;;
            esac
        done
  fi
}

function enable_workload_identity_if_not_already {

  # currently luna only supports gke zonal cluster
  enabled_at_cluster=$(_gcloud container clusters describe "$CLUSTER_NAME" --zone "$CLUSTER_ZONE" --format='value(workloadIdentityConfig.workloadPool)')
  if [ -n "$enabled_at_cluster" ]; then
    echo "Workload Identity already enabled at cluster: $CLUSTER_NAME"
  else
    ask_permission_to_enable_workload_identity
    echo "Enabling Workload Identity for Cluster $CLUSTER_NAME"
    _gcloud container clusters update "$CLUSTER_NAME" --zone "$CLUSTER_ZONE" --workload-pool "$PROJECT_ID".svc.id.goog
  fi

  enabled_at_nodepool=$(_gcloud container node-pools describe "$LUNA_NODEPOOL" --cluster "$CLUSTER_NAME" --zone "$CLUSTER_ZONE" --format='value(config.workloadMetadataConfig.mode)')
  if [ -n "$enabled_at_nodepool" ]; then
    echo "Workload Identity already enabled at nodepool: $LUNA_NODEPOOL"
  else
    ask_permission_to_enable_workload_identity
    echo "Enabling Workload Identity for Node Pool $LUNA_NODEPOOL"
    _gcloud container node-pools update "$LUNA_NODEPOOL" --cluster "$CLUSTER_NAME" --zone "$CLUSTER_ZONE" --workload-metadata=GKE_METADATA
  fi
}

enable_workload_identity_if_not_already
create_namespace_if_not_exist
create_gsa_if_not_exist "$SYSTEM_GSA"
add_gsa_roles "$PROJECT_ID" "$SYSTEM_GSA"
bind_gsa_and_ksa "$SYSTEM_GSA" "$LUNA_MANAGER_SERVICE_ACCOUNT_NAME"

readonly helm_values="./${CLUSTER_NAME}_values.yaml"

cat <<EOF >"$helm_values"
cloudProvider: gcp
clusterID: ${CLUSTER_NAME}
gcp:
  clusterLocation: ${CLUSTER_ZONE}
  serviceAccount: ${SYSTEM_GSA}@${PROJECT_ID}.iam.gserviceaccount.com
labels: "elotl-luna=true"
imageRepo: ${LUNA_MANAGER_REPO}
webhookRepo: ${LUNA_WEBHOOK_REPO}
serviceAccount:
  prefix: ${service_account_prefix}
EOF
